@@ -1,5 +1,5 @@
 /* Command processing for GNU Make.
-Copyright (C) 1988, 1989, 1991, 1992, 1993 Free Software Foundation, Inc.
+Copyright (C) 1988, 1989, 1991, 1992, 1993, 1994 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -331,18 +331,9 @@ fatal_error_signal (sig)
 {
   handling_fatal_signal = 1;
 
+  /* Set the handling for this signal to the default.
+     It is blocked now while we run this handler.  */
   signal (sig, SIG_DFL);
-#ifdef	POSIX
-  {
-    sigset_t set;
-    sigemptyset (&set);
-    (void) sigprocmask (SIG_SETMASK, &set, (sigset_t *) 0);
-  }
-#else
-#ifdef	HAVE_SIGSETMASK
-  (void) sigsetmask (0);
-#endif
-#endif
 
   /* A termination signal won't be sent to the entire
      process group, but it means we want to kill the children.  */
@@ -390,12 +381,55 @@ fatal_error_signal (sig)
        cause a core dump.  Just exit instead.  */
     exit (1);
 
-  /* Signal the same code; this time it will really be fatal.  */
+  /* Signal the same code; this time it will really be fatal.  The signal
+     will be unblocked when we return and arrive then to kill us.  */
   if (kill (getpid (), sig) < 0)
-    /* It shouldn't return, but if it does, die anyway.  */
     pfatal_with_name ("kill");
 }
 
+/* Delete FILE unless it's precious or not actually a file (phony),
+   and it has changed on disk since we last stat'd it.  */
+
+static void
+delete_target (file, on_behalf_of)
+     struct file *file;
+     char *on_behalf_of;
+{
+  struct stat st;
+
+  if (file->precious || file->phony)
+    return;
+
+#ifndef NO_ARCHIVES
+  if (ar_name (file->name))
+    {
+      if (ar_member_date (file->name) != file->last_mtime)
+	{
+	  if (on_behalf_of)
+	    error ("*** [%s] Archive member `%s' may be bogus; not deleted",
+		   on_behalf_of, file->name);
+	  else
+	    error ("*** Archive member `%s' may be bogus; not deleted",
+		   file->name);
+	}
+      return;
+    }
+#endif
+
+  if (stat (file->name, &st) == 0
+      && S_ISREG (st.st_mode)
+      && (time_t) st.st_mtime != file->last_mtime)
+    {
+      if (on_behalf_of)
+	error ("*** [%s] Deleting file `%s'", on_behalf_of, file->name);
+      else
+	error ("*** Deleting file `%s'", file->name);
+      if (unlink (file->name) < 0)
+	perror_with_name ("unlink: ", file->name);
+    }
+}
+     
+
 /* Delete all non-precious targets of CHILD unless they were already deleted.
    Set the flag in CHILD to say they've been deleted.  */
 
@@ -403,36 +437,17 @@ void
 delete_child_targets (child)
      struct child *child;
 {
-  struct stat st;
   struct dep *d;
 
   if (child->deleted)
     return;
 
-  /* Delete the file unless it's precious or not actually a file (phony).  */
-  if (!child->file->precious && !child->file->phony
-      && stat (child->file->name, &st) == 0
-      && S_ISREG (st.st_mode)
-      && (time_t) st.st_mtime != child->file->last_mtime)
-    {
-      error ("*** Deleting file `%s'", child->file->name);
-      if (unlink (child->file->name) < 0)
-	perror_with_name ("unlink: ", child->file->name);
-    }
+  /* Delete the target file if it changed.  */
+  delete_target (child->file, (char *) 0);
 
-  /* Also remove any non-precious targets listed
-     in the `also_make' member.  */
+  /* Also remove any non-precious targets listed in the `also_make' member.  */
   for (d = child->file->also_make; d != 0; d = d->next)
-    if (!d->file->precious && !d->file->phony)
-      if (stat (d->file->name, &st) == 0
-	  && S_ISREG (st.st_mode)
-	  && (time_t) st.st_mtime != d->file->last_mtime)
-	{
-	  error ("*** [%s] Deleting file `%s'", child->file->name,
-		 d->file->name);
-	  if (unlink (d->file->name) < 0)
-	    perror_with_name ("unlink: ", d->file->name);
-	}
+    delete_target (d->file, child->file->name);
 
   child->deleted = 1;
 }
