@@ -1,5 +1,5 @@
 /* Variable function expansion for GNU Make.
-Copyright (C) 1988, 1989, 1991, 1992, 1993 Free Software Foundation, Inc.
+Copyright (C) 1988, 1989, 1991, 1992, 1993, 1994 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -51,8 +51,23 @@ subst_expand (o, text, subst, replace, slen, rlen, by_word, suffix_only)
       return o;
     }
 
-  while ((p = sindex (t, 0, subst, slen)) != 0)
+  do
     {
+      if ((by_word | suffix_only) && slen == 0)
+	/* When matching by words, the empty string should match
+	   the end of each word, rather than the end of the whole text.  */
+	p = end_of_token (next_token (t));
+      else
+	{
+	  p = sindex (t, 0, subst, slen);
+	  if (p == 0)
+	    {
+	      /* No more matches.  Output everything left on the end.  */
+	      o = variable_buffer_output (o, t, strlen (t));
+	      return o;
+	    }
+	}
+
       /* Output everything before this occurrence of the string to replace.  */
       if (p > t)
 	o = variable_buffer_output (o, t, p - t);
@@ -73,11 +88,7 @@ subst_expand (o, text, subst, replace, slen, rlen, by_word, suffix_only)
 
       /* Advance T past the string to be replaced.  */
       t = p + slen;
-    }
-
-  /* Output everything left on the end.  */
-  if (*t != '\0')
-    o = variable_buffer_output (o, t, strlen (t));
+    } while (*t != '\0');
 
   return o;
 }
@@ -337,8 +348,18 @@ expand_function (o, function, text, end)
 	if (argv == 0)
 	  break;
 
+	/* Using a target environment for `shell' loses in cases like:
+	   	export var = $(shell echo foobie) 
+	   because target_environment hits a loop trying to expand $(var)
+	   to put it in the environment.  This is even more confusing when
+	   var was not explicitly exported, but just appeared in the
+	   calling environment.  */
+#if 1
+	envp = environ;
+#else
 	/* Construct the environment.  */
 	envp = target_environment ((struct file *) 0);
+#endif
 
 	/* For error messages.  */
 	if (reading_filename != 0)
@@ -372,9 +393,11 @@ expand_function (o, function, text, end)
 	    /* Free the storage only the child needed.  */
 	    free (argv[0]);
 	    free ((char *) argv);
+#if 0
 	    for (i = 0; envp[i] != 0; ++i)
 	      free (envp[i]);
 	    free ((char *) envp);
+#endif
 
 	    /* Record the PID for reap_children.  */
 	    shell_function_pid = pid;
@@ -515,7 +538,7 @@ expand_function (o, function, text, end)
 	  {
 	    if (wordi >= nwords - 1)
 	      {
-		nwords += 5;
+		nwords *= 2;
 		words = (char **) xrealloc ((char *) words,
 					    nwords * sizeof (char *));
 	      }	
