@@ -1,4 +1,5 @@
-/* Copyright (C) 1988, 1989, 1990, 1991 Free Software Foundation, Inc.
+/* Argument parsing and main program of GNU Make.
+Copyright (C) 1988, 89, 90, 91, 94 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -454,9 +455,6 @@ main (argc, argv, envp)
   /* Initialize the directory hashing code.  */
   init_dir ();
 
-  /* Set up to access user data (files).  */
-  user_access ();
-
   /* Figure out where this program lives.  */
 
   if (argv[0] == 0)
@@ -472,6 +470,9 @@ main (argc, argv, envp)
 	++program;
     }
 
+  /* Set up to access user data (files).  */
+  user_access ();
+
   /* Figure out where we are.  */
 
   if (getcwd (current_directory, GET_PATH_MAX) == 0)
@@ -861,7 +862,7 @@ main (argc, argv, envp)
 	  {
 	    register struct file *f = d->file;
 	    if (f->double_colon)
-	      do
+	      for (f = f->double_colon; f != NULL; f = f->prev)
 		{
 		  if (f->deps == 0 && f->cmds != 0)
 		    {
@@ -889,9 +890,7 @@ main (argc, argv, envp)
 
 		      break;
 		    }
-		  f = f->prev;
 		}
-	      while (f != NULL);
 	    if (f == NULL || !f->double_colon)
 	      {
 		if (makefile_mtimes == 0)
@@ -978,7 +977,7 @@ main (argc, argv, envp)
 	    if (any_remade)
 	      goto re_exec;
 	    else if (any_failed)
-	      die (1);
+	      die (2);
 	    else
 	      break;
 	  }
@@ -1095,8 +1094,12 @@ main (argc, argv, envp)
 	    /* Updated successfully.  */
 	    status = 0;
 	    break;
-	  case 1:
+	  case 2:
 	    /* Updating failed.  */
+	    status = 2;
+	    break;
+	  case 1:
+	    /* We are under -q and would run some commands.  */
 	    status = 1;
 	    break;
 	  default:
@@ -1419,7 +1422,7 @@ positive integral argument",
 		   buf, cs->description);
 	}
 
-      die (bad);
+      die (bad ? 2 : 0);
     }
 }
 
@@ -1705,7 +1708,7 @@ print_version ()
     printf ("-%s", remote_description);
 
   printf (", by Richard Stallman and Roland McGrath.\n\
-%sCopyright (C) 1988, 89, 90, 91, 92, 93 Free Software Foundation, Inc.\n\
+%sCopyright (C) 1988, 89, 90, 91, 92, 93, 94 Free Software Foundation, Inc.\n\
 %sThis is free software; see the source for copying conditions.\n\
 %sThere is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n\
 %sPARTICULAR PURPOSE.\n\n", precede, precede, precede, precede);
