@@ -1,4 +1,5 @@
-/* Copyright (C) 1988, 1989, 1991, 1992, 1993 Free Software Foundation, Inc.
+/* Definitions for using pattern rules in GNU Make.
+Copyright (C) 1988, 1989, 1991, 1992, 1993 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -41,6 +42,7 @@ extern struct rule *last_pattern_rule;
 extern unsigned int num_pattern_rules;
 
 extern unsigned int max_pattern_deps;
+extern unsigned int max_pattern_targets;
 extern unsigned int max_pattern_dep_length;
 
 extern struct file *suffix_file;
